﻿/**
 * @file cpld_protocol.h
 * @brief Library for communication with CPLD header
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CPLD_PROTOCOL_H_
#define CPLD_PROTOCOL_H_

#include <stdbool.h>
#include <stdint.h>

#define CPLD_ADDR_VERSION				0x0
#define CPLD_ADDR_RELAY_CONTROL			0x1
#define CPLD_ADDR_RELAY_STATUS			0x2
#define CPLD_ADDR_ALARMPIN0_MASK		0x3
#define CPLD_ADDR_ALARMPIN1_MASK		0x4
#define CPLD_ADDR_ALARMPIN_TRIGGERED	0x5
#define CPLD_ADDR_CONTROL				0x6
#define CPLD_ADDR_SCRATCHPAD			0x7
#define CPLD_ADDR_PWMENABLE 			0x8
#define CPLD_ADDR_PWMVALUE   			0x9

#define CPLD_ADDR_ALARMPIN1_TRIGGERED	0x01
#define CPLD_ADDR_ALARMPIN2_TRIGGERED   0x02

#define CPLD_AFTER_RESET_VALUE			0x00
#define CPLD_STARTED_VALUE				0x01

#define CPLD_CONTROL_CLEAR_INT			(1 << 6)
#define CPLD_CONTROL_ERROR_LED			(1 << 5)
#define CPLD_CONTROL_ENABLE_TMCB_MP		(1 << 4)

#define CPLD_SPI_SPEED					10000000
#define CPLD_SPI_READ_OFFSET			0x80

#define CPLD_ERROR_LED_CLEAR_TIME_S		2

void	CPLD_init(void);

uint8_t readRegister(uint8_t address);
uint8_t readHalfRegister(uint8_t address, uint8_t half);
void	writeRegister(uint8_t address, uint8_t value);
void	writeHalfRegister(uint8_t address, uint8_t half, uint8_t value);
void	setAddress( uint8_t address, uint8_t half );

uint8_t getCPLDversion(void);
uint8_t getAlarmPin0Mask(void);
uint8_t getAlarmPin1Mask(void);
void	setAlarmPin0Mask(uint8_t value);
void	setAlarmPin1Mask(uint8_t value);
uint8_t getAlarmPinsTriggered(void);

void	setRelaysState(uint8_t state);
uint8_t getRelaysState(void);
void	clearCpldInterrupt(void);
void	setCpldErrorLED(bool on);
void	clearAlarms(void);
void	refreshCpldErrorLED(void);
void	enableTmcbMP(void);
void	disableTmcbMP( void );

void	setTMCBPresence(uint8_t state);
uint8_t getTMCBPresence(void);

uint8_t getCPLDscratchpad(void);
void	setCPLDscratchpad(uint8_t value);


#endif /* CPLD_PROTOCOL_H_ */