﻿/**
 * @file i2c_lib.C
 * @brief CPU EEPROM support library
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef EEPROM_H_
#define EEPROM_H_

#include <stdint.h>
#include <stdbool.h>

#define EEPROM_INIT_VALUE (uint8_t) 0x55

#define EEPROM_ADDR_INIT			(uint8_t *)  0
#define EEPROM_ADDR_LIMIT_MASK		(uint8_t *)  1
#define EEPROM_ADDR_LIMIT_CHANNEL0	(uint32_t *) 4
#define EEPROM_ADDR_LIMIT_CHANNEL1	(uint32_t *) 8
#define EEPROM_ADDR_LIMIT_CHANNEL2	(uint32_t *) 12
#define EEPROM_ADDR_LIMIT_CHANNEL3	(uint32_t *) 16
#define EEPROM_ADDR_LIMIT_CHANNEL4	(uint32_t *) 20
#define EEPROM_ADDR_LIMIT_CHANNEL5	(uint32_t *) 24
#define EEPROM_ADDR_LIMIT_CHANNEL6	(uint32_t *) 28
#define EEPROM_ADDR_LIMIT_CHANNEL7	(uint32_t *) 32

#define EEPROM_ADDR_JUMPER_PRESENCE	(uint8_t *) 63	//

#define EEPROM_ADDR_ADC_CH0_OFFSET	(uint8_t *) 64	//
#define EEPROM_ADDR_ADC_CH1_OFFSET	(uint8_t *) 65	//
#define EEPROM_ADDR_ADC_CH2_OFFSET	(uint8_t *) 66	//
#define EEPROM_ADDR_FAN_SPEED_LOW	(uint8_t *) 67	//
#define EEPROM_ADDR_FAN_SPEED_HIGH	(uint8_t *) 68	//
#define EEPROM_ADDR_FAN_SPEED_MODE	(uint8_t *) 69	//
#define EEPROM_ADDR_FAN_SHUNT		(uint8_t *) 70	//

#define FAN_SPEED_MODE_VOLTAGE					2
#define FAN_SPEED_MODE_RPM						1
#define FAN_SPEED_MODE_UNDEFINED				0

#define EEPROM_ADDR_BOX_NAME		(uint8_t *)	 100
#define EEPROM_ADDR_INV_NO			(uint8_t *)	 200

#define EEPROM_ADDR_UNCONDITIONAL_MASK	(uint8_t *) 300
#define EEPROM_ADDR_HOTPLUG_MASK	(uint8_t *) 301

#define EEPROM_CHNAME_FIRST			(uint8_t *) 305
#define EEPROM_CHNAME_LEN			20
#define EEPROM_CHNAME(x)			(uint8_t *) (EEPROM_CHNAME_FIRST + x * EEPROM_CHNAME_LEN)

#define EEPROM_MAGICBYTE_ADDR		                    (uint8_t *)  0x7FF // End of EEPROM
#define EEPROM_MAGICBYTE_FWUPGRADE_REQUESTED_VALUE		(uint8_t)    0x56
#define EEPROM_MAGICBYTE_FWREADY_VALUE		            (uint8_t)    0x5A // changed to be identical to MMC Bootloader

#define EEPROM_STR_LEN				64

#define EEPROM_UNCONDITIONAL_MASK_DEFAUALT  0xFF
#define EEPROM_HOTPLUG_MASK_DEFAULT			0x00

void initEEPROM(void);
void eeprom_write_upgrade_requested(void);

void eeprom_update_cur_limit(unsigned channel);

void eeprom_write_box_name(char * name);
void eeprom_write_inv_no(char * number);
void eeprom_read_box_name(char * name);
void eeprom_read_inv_no(char * nr);

void eeprom_write_unconditional_mask(uint8_t mask);
void eeprom_write_hotplug_mask(uint8_t mask);
uint8_t eeprom_read_unconditional_mask(void);
uint8_t eeprom_read_hotplug_mask(void);
bool eeprom_chname_ch_valid(char c);
void eeprom_chname_write(int channel, char *name);
void eeprom_chname_read(int channel, char *name);
#endif /* EEPROM_H_ */