﻿/*
 * errors.c
 *
 * Created: 2015-08-25 13:37:40
 *  Author: Paweł
 */ 
#include "errors.h"

#include <asf.h>
#include <stdio.h>
#include "usart_lib.h"

TransmissionStatistics transmissionStatistics = {0};

void increment_spi_cpld( int8_t status )
{
	transmissionStatistics.totalSPICpld++;
	if (status != STATUS_OK)
	{
		#ifdef DBG_LEVEL_ERROR
		printf("%%%%%% Error in SPI communication with CPLD: %d\r\n", status);
		#endif
		transmissionStatistics.errorsSPICpld++;
	}
}

void print_transmission_statistics()
{
	printPString("Transmission statistics (errors/total):\r\n");
	printf("i2c internal: %u/%u\r\n", transmissionStatistics.errorsTWIinternal, transmissionStatistics.totalTWIinternal);
	printf("SN EEPROM: %u/%u\r\n", transmissionStatistics.errorsSnEeprom, transmissionStatistics.totalSnEeprom);
	for (int i = 0; i < CHANNEL_COUNT; ++i)
		printf("power monitor %d: %u/%u\r\n", i, transmissionStatistics.errorsPowerMonitor[i], transmissionStatistics.totalPowerMonitor[i]);
	printf("i2c external: %u/%u\r\n", transmissionStatistics.errorsTWIexternal, transmissionStatistics.totalTWIexternal);
	printf("fan controller chip: %u/%u\r\n", transmissionStatistics.errorsFanController, transmissionStatistics.totalFanController);
	printf("fan board cpu: %u/%u\r\n", transmissionStatistics.errorsFanboardCPU, transmissionStatistics.totalFanboardCPU);
	printf("tmcb mgmt: %u/%u\r\n", transmissionStatistics.errorsTmcbMgmt, transmissionStatistics.totalTmcbMgmt);
	printf("spi cpld: %u/%u\r\n", transmissionStatistics.errorsSPICpld, transmissionStatistics.totalSPICpld);
}

void increment_i2c_internal( void )
{
	transmissionStatistics.totalTWIinternal++;
}

void error_i2c_internal( void )
{
	transmissionStatistics.errorsTWIinternal++;
}

void increment_i2c_eeprom( int8_t status )
{
	increment_i2c_internal();
	transmissionStatistics.totalSnEeprom++;
	if (status != STATUS_OK)
	{
		#ifdef DBG_LEVEL_ERROR
		printf("%%%%%% Error in i2c communication with UID EEPROM: %d\r\n", status);
		#endif
		error_i2c_internal();
		transmissionStatistics.errorsSnEeprom++;
	}
}

void increment_i2c_power_monitor( uint8_t number, int8_t status )
{
	if (number < CHANNEL_COUNT)
	{
		increment_i2c_internal();
		transmissionStatistics.totalPowerMonitor[number]++;
		if (status != STATUS_OK)
		{
			#ifdef DBG_LEVEL_ERROR
			///TODO remove for test on good hw
// 			printf("!ERR: Error in i2c communication with Power Monitor #%d: %d\r\n", number, status);
			#endif
			error_i2c_internal();
			transmissionStatistics.errorsPowerMonitor[number]++;
		}
	}
}


void increment_i2c_external( void )
{
	transmissionStatistics.totalTWIexternal++;
}


void error_i2c_external( void )
{
	transmissionStatistics.errorsTWIexternal++;
}

void increment_fan_controller( int8_t status )
{
		increment_i2c_external();
		transmissionStatistics.totalFanController++;
		if (status != STATUS_OK)
		{
			#ifdef DBG_LEVEL_ERROR
			printf("%%%%%% Error in i2c communication with Fan controller IC: %d\r\n", status);
			#endif
			error_i2c_external();
			transmissionStatistics.errorsFanController++;
		}
}

void increment_fanboard_cpu( int8_t status )
{
	increment_i2c_external();
	transmissionStatistics.totalFanboardCPU++;
	if (status != STATUS_OK)
	{
		#ifdef DBG_LEVEL_ERROR
		printf("%%%%%% Error in i2c communication with Fan board MCU: %d\r\n", status);
		#endif
		error_i2c_external();
		transmissionStatistics.errorsFanboardCPU++;
	}
}

void increment_tmcb_mgmt( int8_t status )
{
	static int8_t statusOld; // old status value
	increment_i2c_external();
	transmissionStatistics.totalTmcbMgmt++;
	if (status != STATUS_OK)
	{
		#ifdef DBG_LEVEL_ERROR
		if (status != statusOld) //disable repeated output
		printf("%%%%%% Error in i2c communication with TMCB board: %d\r\n", status);		
		statusOld = status; // save old status value
		#endif
		error_i2c_external();
		transmissionStatistics.errorsTmcbMgmt++;
	}
	else
	{
		statusOld = status; // save old status value
	}
}
