/*
* fan_current_monitor_lib.c
*
* Created: 09.02.2015 14:10:07
*  Author: schystia
*/

#include "i2c_lib.h"
#include "twim.h"
#include "mgmt_protocol_master.h"
#include "usart_lib.h"
#include "telnet_protocol.h"
#include "power_monitor.h"

#include "fan_current_monitor_lib.h"
#include "cpld_protocol.h"
#include "fan_lib.h"
#include "eeprom.h"
#include <asf.h>
#include <util/delay.h>
#include <stdio.h>
#include <avr/eeprom.h>

uint8_t portName = 0; // selected port to modify
float currentScaleVal_mA = 0;



int FREDFAN_Board_CheckPresence(void)
{
	uint8_t reg;
	uint8_t read_data[2] = {0,0};
	reg = CMD_TWI_CHECK_PRESENCE;
	
	getCurrentScale();
	
	if ( i2c_read(&EXTERNAL_I2C, FREDFAN_ADDR,  &reg, 1, read_data, 2) == TWI_SUCCESS) {		
		printf("%% FRED Fan Ver. %d Rev. %d\r\n", read_data[0], read_data[1]);
		//adcCalibrate();
		return STATUS_OK;
	}
	else {
		printPString("%%%% FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

void readFanCurrentAverageAll(void){	// read all 3 ADC Channels
										// ATmega88PA samples N ADC values for each channel and calculates average value
	uint8_t addr, reg;
	int data;
	uint8_t shift, i;
	uint8_t read_data[4] = {0,0,0,0};
	float current;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_ADC_ALL;
	
	if(i2c_read(&EXTERNAL_I2C, addr, &reg, 1, read_data, 4) == TWI_SUCCESS)
	{
		shift = 2;
		for(i = 0; i < 3 ; i++) {
			data = read_data[i] + ((read_data[3] << shift) & 0x0300);
			shift += 2;
			current = data * currentScaleVal_mA;
			printf("Average current value of FAN[%d] = %d mA\r\n", i, (int)current);
		}
	}
	else
	{
		printPString("%% Failure reading values!\r\n");
	}
}

int adcCalibrate(void)
{
	uint8_t adcChnlNum;
	uint8_t addr, reg;
	int data;
	uint8_t read_data[4] = {0,0,0,0};
		
	//setFanSpeed(0);
	
	uint8_t adc_channel[3] = {ADC_CH0, ADC_CH1, ADC_CH2};
	uint8_t *adc_channel_offset[3] = {EEPROM_ADDR_ADC_CH0_OFFSET, EEPROM_ADDR_ADC_CH1_OFFSET, EEPROM_ADDR_ADC_CH2_OFFSET};
	
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_ADC_ALL;
	
	for (adcChnlNum=0; adcChnlNum < 3; adcChnlNum++)
	{
		
		if(i2c_read(&EXTERNAL_I2C, addr, &reg, 1, read_data, 4) == TWI_SUCCESS)
		{
			data = read_data[adc_channel[adcChnlNum]] + ((read_data[3] << (2*(adc_channel[adcChnlNum]+1))) & 0x0300);
			eeprom_write_byte(adc_channel_offset[adcChnlNum] /*(uint8_t *) 64+adcChnlNum*/, (uint8_t) data);
			
		}
		else
		{
			return ERR_IO_ERROR;
		}
	}
	
	//setFanSpeed(eeprom_read_byte(EEPROM_ADDR_FAN_SPEED));
	return STATUS_OK;
}

int readFanCurrent_mA( uint8_t fan, unsigned *current )
{
	uint8_t addr, reg;
	int data;
	uint8_t read_data[4] = {0,0,0,0};
		
	uint8_t fan_channel[3] = {FAN1_CURRENT, FAN2_CURRENT, FAN3_CURRENT};
	//uint8_t *adc_channel_offset[3] = {EEPROM_ADDR_ADC_CH2_OFFSET, EEPROM_ADDR_ADC_CH1_OFFSET, EEPROM_ADDR_ADC_CH0_OFFSET};
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_ADC_ALL;
	
	if(i2c_read(&EXTERNAL_I2C, addr, &reg, 1, read_data, 4) == TWI_SUCCESS)
	{
		//data = read_data[fan] + ((read_data[3] << (2*(fan+1))) & 0x0300);
		data = read_data[fan_channel[fan]] + ((read_data[3] << (2*(fan_channel[fan]+1))) & 0x0300);
		*current = (int) (data /*- eeprom_read_byte(adc_channel_offset[fan])*/ * currentScaleVal_mA);
		return STATUS_OK;
	}
	else
	{
		return ERR_IO_ERROR;
	}
}

void setFanShuntVal(uint8_t shuntVal)
{
	eeprom_write_byte(EEPROM_ADDR_FAN_SHUNT, shuntVal);
	getCurrentScale();
	
	printf("%% Value of the fan shunt is %d mOhm\r\n", getFanShuntVal());
}

uint8_t getFanShuntVal(void)
{
	uint8_t shuntVal;
	shuntVal = eeprom_read_byte(EEPROM_ADDR_FAN_SHUNT);
	if ((shuntVal <= 0x00) || (shuntVal >= 0xFF))
	{
		printPString("%% FAN current shunt is not set\r\n");
	}
	return shuntVal;
}

void getCurrentScale(void)
{
	float shuntVal;
	shuntVal = getFanShuntVal() / 1000.0; // Ohm
	if ((shuntVal > 0x00) && (shuntVal < 0xFF))
	{
		currentScaleVal_mA = ADC_V_REF / ADC_RES / (CURRENT_SENSOR_SCALE * shuntVal);
							//(1100 mV / 2^10) / (10 * 0,X Ohm)
	}
	else currentScaleVal_mA = ADC_V_REF / ADC_RES / (CURRENT_SENSOR_SCALE * FAN_SHUNT_50);
	
}
int resetTimeToErr(void) // watch dog
{
	static uint8_t printEnable = 1;
	//char message[80];
	uint8_t addr, reg, data;
	uint8_t alarmStatus, alarmStatusNew, watchDogAlarmStatus;
	static uint8_t watchDogAlarmStatusPrev = 1;
	addr = FREDFAN_ADDR;
	reg = CMD_RESET_ERR_TIME;
	data = 0;
	
	if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		readFanAlarmStatusReg(&alarmStatus);
		if(alarmStatus) {
			//sprintf(message, "Alarm Status Register = 0x%X\r\n", status);
			//printString(message);
			readFanAlarmStatusReg(&alarmStatusNew);
			if(!alarmStatusNew) {
				if(forceFanOff) setFanSpeedOpenLoop(FAN_SPEED_V_MODE_ZERO, RESULT_QUIET);
				else set_previos_fan_speed();
			}
		}
		readFanBoardWatchDogStatusReg(&watchDogAlarmStatus);
		//sprintf(message, "WD Status Register = 0x%X\r\n", watchDogAlarmStatus);
		//printString(message);
		if(watchDogAlarmStatus != watchDogAlarmStatusPrev)
		{		
			// Set Fan Speed
			//printString("Set previous speed after WD-error\r\n");
		}
		watchDogAlarmStatusPrev = watchDogAlarmStatus;
		printEnable = 1;
		return STATUS_OK;
	}
	else
	{
		if (printEnable == 1)
		printPString("%% Access to ATmega88PA error!\r\nWatchdog reset error!\r\n");
		printEnable = 0;
		return ERR_IO_ERROR;
	}
}

void set_previos_fan_speed(void)
{	
	uint16_t speed_rpm;
	if (eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_MODE) == FAN_SPEED_MODE_VOLTAGE)
	{
		setFanSpeedOpenLoop(eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_LOW), RESULT_QUIET);
	}
	else if (eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_MODE) == FAN_SPEED_MODE_RPM)
	{
		speed_rpm = eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_LOW);
		speed_rpm = speed_rpm | (eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_HIGH) << 8);
		setFanSpeedOpenLoopRpm(speed_rpm, RESULT_QUIET);
	}
	else// if (eeprom_read_byte(EEPROM_ADDR_FAN_SPEED_MODE) == FAN_SPEED_MODE_UNDEFINED)
	{
		setFanSpeedOpenLoop(FAN_SPEED_V_MODE_MAX, RESULT_QUIET);
	}
	
}

//int calibrate_fanbord_ADCs(void) // ???????????
//{
	//uint8_t addr, reg, data;
	//addr = FREDFAN_ADDR;
	//reg = CMD_TWI_ADC_CAL;
	//data = 0;
	//
	//if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	//{
		//printString("Fan Board ADCs are calibrated\r\n");
		//return STATUS_OK;
	//}
	//else {
		//printString("Access to ATmega88PA error!\r\nWatchdog reset error!\r\n");
		//return ERR_IO_ERROR;
	//}
//}

int setAverage(uint8_t inptVal) // set average for ADC (number of samples)
{
	uint8_t addr, reg, data;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_AVERAGE;
	data = inptVal;
	
	if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		return STATUS_OK;
	}
	else {
		printPString("%% FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

void selectPort(uint8_t inptVal)
{
	portName = inptVal;
}

int setPortDirection(uint8_t port, uint8_t portDir)
{
	uint8_t addr, reg;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_PORT_DIR;
	uint8_t transm_data[2] = {0};
	
	transm_data[0] = portName;
	transm_data[1] = portDir;
	
	if (portName != 0) {
		if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, transm_data, 2) == TWI_SUCCESS)
		{
			printPString("Success\r\n");
			return STATUS_OK;
		}
		else {
			printPString("FRED Fan board not detected!\r\n");
			return ERR_IO_ERROR;
		}
	}
	else
	{
		printPString("You should first select the port!\r\n");
		return ERR_IO_ERROR;
	}
}

int setPortLevel(uint8_t port, uint8_t portLev)
{
	uint8_t addr, reg;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_PORT_LEV;
	uint8_t transm_data[2] = {0,0};
	
	transm_data[0] = port;
	transm_data[1] = portLev;

	if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, transm_data, 2) == TWI_SUCCESS)
	{
		return STATUS_OK;
	}
	else {
		printPString("%% FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

int readPortLevel(uint8_t port)
{
	uint8_t addr, reg, data;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_PORT_LEV;
	data = port;
	
	if(i2c_write(&EXTERNAL_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		if(i2c_read(&EXTERNAL_I2C, addr, &reg, 1, &data, 1) == TWI_SUCCESS)
		{
			printf("0x%X\r\n", data);
			return STATUS_OK;
		}
		else {
			printPString("%% FRED Fan board not detected!\r\n");
			return ERR_IO_ERROR;
		}
	}
	return -1;
}

uint8_t readFanBoardWatchDogStatusReg(uint8_t *value) //
{
	uint8_t addr, reg, data;
	//char message[80];
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_WD_REG;
	
	if(i2c_read(&EXTERNAL_I2C, addr, &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		*value = data;
		//sprintf(message, "Alarm Status Register = 0x%X\r\n", data); //use this to show value of Alarm Status Register
		//printString(message);
		return STATUS_OK;
	}
	else {
		//printString("FAN board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}