/*
 * fan_current_monitor_lib.h
 *
 * Created: 09.02.2015 14:10:17
 *  Author: schystia
 */ 

#ifndef FAN_CURRENT_MONITOR_LIB_H_
#define FAN_CURRENT_MONITOR_LIB_H_
#include <stdbool.h>

//#define ADC_SCALE_mV				1.07421875F		// (1100 mV / 2^10)
//#define CURRENT_SCALE_50			2.1484375F		// (1100 mV / 2^10) / (10 * 0,05 Ohm)	[50mOhm]
//#define CURRENT_SCALE_200			0.537109375F	// (1100 mV / 2^10) / (10 * 0,2 Ohm)	[200mOhm]
//#define CURRENT_SCALE_500			0.21484375F		// (1100 mV / 2^10) / (10 * 0,5 Ohm)	[500mOhm]
//#define FAN_SHUNT_200				200.0			// 200 mOhm
#define FAN_SHUNT_50				50./1000.		// 50 mOhm
#define ADC_V_REF					1100.
#define ADC_RES						1024.
#define CURRENT_SENSOR_SCALE		10.


#define FREDFAN_ADDR				0x0C >> 1 // FRED Fan board address

#define ADC_CH0						0x00	//
#define ADC_CH1						0x01	//
#define ADC_CH2						0x02	//

#define FAN3_CURRENT				ADC_CH0
#define FAN2_CURRENT				ADC_CH1
#define FAN1_CURRENT				ADC_CH2


//#define CMD_TWI_WRITE				0x10
#define CMD_TWI_READ_ADC			0x20	// read only 1 ADC channel
//#define CMD_TWI_READ_ADC_AVRG		0x21	// read average only for 1 ADC channel (calculated by ATmega88PA)
#define CMD_TWI_READ_ADC_ALL		0x22	// read average for all channels (calculated by ATmega88PA)
#define CMD_TWI_READ_VAL			0x23	// read value from ATmega88PA (Test ONLY!!!)
#define CMD_TWI_CHECK_PRESENCE		0x24	// read FRED Fan board version and revision
#define CMD_TWI_SET_AVERAGE			0x25	// set number of ADC samples
#define CMD_TWI_SET_PORT_DIR		0x26	// set port direction /setPortDirection()
#define CMD_TWI_SET_PORT_LEV		0x27	// set port level /setPortLevel()
#define CMD_TWI_READ_PORT_LEV		0x28	// read Port B level /readPortLevel()
#define CMD_TWI_READ_WD_REG			0x29	// read watchdog status register


#define CMD_RESET_ERR_TIME			0x30	// reset error time

#define FB_PORTB_					0x41	// transmit value for PORTB
#define FB_PORTD_					0x42	// transmit value for PORTD
#define _PORTC_						0x43	// transmit value for PORTC


int FREDFAN_Board_CheckPresence(void);
int resetTimeToErr(void);
void set_previos_fan_speed(void);

int readFanCurrent_mA(uint8_t fan, unsigned *current);
void readFanCurrentAverage(uint8_t adcChnl);
void readFanCurrentAverageAll(void);
void setFanShuntVal(uint8_t shuntVal);
uint8_t getFanShuntVal(void);
void getCurrentScale(void);
int setAverage(uint8_t inptVal);

void selectPort(uint8_t inptVal);
int setPortDirection(uint8_t port, uint8_t portDir);
int setPortLevel(uint8_t port, uint8_t portLev);
int readPortLevel(uint8_t port);

int adcCalibrate(void);
//int calibrate_fanbord_ADCs(void);
uint8_t readFanBoardWatchDogStatusReg(uint8_t *value);

#endif /* FAN_CURRENT_MONITOR_LIB_H_ */




