﻿/**
 * @file usart_lib.h
 * @brief FRED hw revision detection
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Piotr Perek (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "hwrevision.h"
#include <asf.h>

FredHwRevision get_hwrevision( void )
{
	uint8_t rev=0;
    
    if (ioport_pin_is_high(REVDET0)) rev |= 0x01;  // FRED2 REV B
    if (ioport_pin_is_high(REVDET1)) rev |= 0x02;  // FRED2 REV C
    if (ioport_pin_is_high(REVDET2)) rev |= 0x04;  // FRED3M REV C
    if (ioport_pin_is_high(REVDET3)) rev |= 0x08;  // FRED3 REV A
    if (ioport_pin_is_high(REVDET4)) rev |= 0x10;  // FRED3 REV B
    if (ioport_pin_is_high(REVDET5)) rev |= 0x20;  // FRED3M REV D
    rev |= 0b11000000; // Fill upper Bits;
    
    if (rev==0b11111111) return FRED2A;
    if (rev==0b11111110) return FRED2B;
    if (rev==0b11111100) return FRED2C;
    if (rev==0b11111000) return FRED3MC;
    if (rev==0b11110100) return FRED3A;
    if (rev==0b11100100) return FRED3B;
    if (rev==0b11000000) return FRED3MD;
    
	return UNKNOWN;
}
