/**
 * @file main.C
 * @brief Fuse Board Application
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
* Include header files for all drivers that have been imported from
* Atmel Software Framework (ASF).
*/
#include <asf.h>
#include <util/delay.h>
#include <ctype.h>
#include <avr/eeprom.h>
#include <stdio.h>

//define to test communication with CPLD via parallel protocol
#ifdef REGISTER_TEST
	#include "register_test.h"
#endif

#include "timer_lib.h"
#include "i2c_lib.h"
#include "usart_lib.h"
#include "power_monitor.h"
#include "telnet_protocol.h"
#include "cpld_protocol.h"
#include "mgmt_protocol_master.h"
#include "eeprom.h"
#include "events.h"
#include "fan_lib.h"
#include "fan_current_monitor_lib.h"



int main (void)
{
	board_init();
	sysclk_init();
	
	timer_init();	
	usart_init();
	
	i2c_reset_bus(&LOCAL_I2C);
	i2c_reset_bus(&EXTERNAL_I2C);
	
	i2c_init(&LOCAL_I2C);
	pmic_init();
	pmic_set_scheduling(PMIC_SCH_ROUND_ROBIN);
	cpu_irq_enable();
	
	i2c_init(&EXTERNAL_I2C);
	//initialize SPI for CPLD
	CPLD_init();
	
	initEEPROM();
	
	initChannelStructures();
	loadPowerMasks();
	//correct order?
	fan_init();
	fflush(stdout);
	FREDFAN_Board_CheckPresence(); //
	checkTmcbJumperPresence();
	fflush(stdout);
	
	//print version information
	printVersion();		
	printPString("\r\n# ");
	
	
	#ifdef REGISTER_TEST
		testRegisters();
	#endif
	uint8_t counter_1000_ms = 0;
	
	while(1)
	{
		if (timer_100ms_flag)
		{ 
			timer_100ms_flag = false;
			++counter_1000_ms;

			processEvents();
			refreshCpldErrorLED();
		}
		if (counter_1000_ms % 10 == 0)
		{
			resetTimeToErr(); //
			control_fan_state();
		#ifdef TEST_MGMT
			testMgmtProtocol();
		#endif
			nop();
		}		
		if (bufferReady)
		{
			processChannels(false);
			parseCommand();
			updateRelays();
			processChannels(false);
		}
	}
}
