﻿/**
 * @file mgmt_protocol_master.h
 * @brief Management protocol between TMCB2 and FRED boards support library header
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef MGMT_PROTOCOL_MASTER_H_
#define MGMT_PROTOCOL_MASTER_H_

//#define TEST_MGMT			0
#define EXTERNAL_I2C			TWIC
#define MANAGE_ADDR			0x60
#define MANAGE_INTERRUPT	TWID_TWIS_vect

//register addresses
#define REG_COUNT					0x64 ///<Number of registers
#define REG_TEST					0x00 ///<Test register
#define REG_POWER_GOOD				0x01 ///<Power good register
#define REG_FPGA_CONTROL			0x02 ///<FPGA control register
#define REG_FPGA_PROM				0x03 ///<FPGA PROM selection register
#define REG_TEMPERATURE				0x04 ///<Temperature readout register
#define REG_FMC						0x05 ///<FMC register
#define REG_TMCB_PIN_VOLTAGE		0x06 ///<SW pins register
#define REG_TEMPERATURE_STATUS		0x07 ///<Temperature status register
#define REG_POWER_OK				0x08 ///<Power OK register
#define REG_FRED_CHANNEL_MASK		0x09 ///<Channel mask for FRED turning channels on and off on hotplug register

///Voltages -  from 0x10 to 0x0A

///First supported voltage
#define REG_VOLTAGE_FIRST			0x10

///Last supported voltage
#define REG_VOLTAGE_LAST			0x0A

//New architecture - 0x20+
#define REG_TMCB_STATUS				0x20
#define REG_BLUE_LED				0x21


//Slave command
#define REG_SLAVE_CMD				0x63

#define REG_POWER_GOOD_PWRGOOD		(1 << 0) ///<Power good
#define REG_POWER_GOOD_PWROK		(1 << 1) ///<Power OK

#define REG_TEMPERATURE_STATUS_OVERTEMP	(1 << 0) ///<Over-temperature

#define REG_FMC_PRESENCE			(1 << 0) ///<FMC presence
#define REG_FMC_POWERGOOD			(1 << 1) ///<FMC power good

#define INT_PIN						ON_OFF_FLIP_FLOP ///<Interrupt pin for slave->master communication

#define SLAVE_COMMAND_CMD(x)		((x >> 8) & 0xFF) ///<Command from slave->master communication register
#define SLAVE_COMMAND_DATA(x)		((x >> 0) & 0xFF) ///<Data from slave->master communication register

#define SLAVE_CMD_HANDLE			0x01 ///<Handle status slave command

#define VOLTAGE_COUNT				8
#define VOLTAGE_MASK_P15VD			(1 << 0)
#define VOLTAGE_MASK_P15VA			(1 << 1)
#define VOLTAGE_MASK_P15VH			(1 << 2)
#define VOLTAGE_MASK_P7VA			(1 << 3)
#define VOLTAGE_MASK_N15VD			(1 << 4)
#define VOLTAGE_MASK_N15VA			(1 << 5)
#define VOLTAGE_MASK_N7VA			(1 << 6)
#define VOLTAGE_MASK_N15VH			(1 << 7)

#define REG_STATUS_AMC_HANDLE		(1 << 0)
#define REG_STATUS_ALLOW_ENABLE		(1 << 1)
#define REG_STATUS_ALLOW_DISABLE	(1 << 2)
#define REG_STATUS_FPGA_CONFIGURED	(1 << 3)
#define REG_STATUS_FPGA_DONE		(1 << 4)

#define REG_BLUE_LED_OFF			0x00
#define REG_BLUE_LED_BLINK			0x01
#define REG_BLUE_LED_ON				0x03

#define REG_FPGA_CONTROL_DISABLE	0x01
#define REG_FPGA_CONTROL_ENABLE		0x02
#define REG_FPGA_CONTROL_RELOAD		0x04

#define TMCB_PRESENCE_HWJUMPER		'j'
#define TMCB_PRESENCE_PRESENT		'p'
#define TMCB_PRESENCE_ABSENT		'a'
#define TMCB_JUMPER_PRESENT			1
#define TMCB_JUMPER_ABSENT			0

typedef enum {Not_configured, Disabling, Configured} FpgaConfigureState;
extern FpgaConfigureState tmcb_fpga_state;

extern bool tmcb_fpga_poweroff_request;
extern bool tmcbJumperState;

int mgmtReadRegister(uint8_t reg, uint16_t *data);
int mgmtWriteRegister(uint8_t reg, uint16_t data);
uint16_t mgmtReadVoltage(uint8_t sensor);

void testMgmtProtocol(void);

void processTmcbMgmt(void);

int getTmcbStatus( uint16_t *status);
int setTmcbBlueLed(uint16_t value);
bool tmcbIsConfigured(void);
bool tmcbJumperOn(void);
bool tmcbHandleOn(void);


void tmcbEnableFpga(void);
void tmcbRequestDisableFpga(void);

bool canPowerUp(void);
bool canPowerDown(void);

void checkTmcbJumperPresence(void);
void writeTmcbJumperMode(char mode);

#endif /* MGMT_PROTOCOL_MASTER_H_ */