﻿/**
 * @file telnet_protocol.c
 * @brief Telnet protocol for Xport
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TELNET_PROTOCOL_H_
#define TELNET_PROTOCOL_H_

#include <stdbool.h>

#define INPUT_BUFFER_SIZE 128
extern volatile char inputBuffer[INPUT_BUFFER_SIZE];
extern volatile int inputBufferCounter;
extern volatile bool bufferReady;

void telnetPutChar(int ch);

void parseCommand(void);
void clearBuffer(void);

void printChannelHeader (void);
void printChannel (int nr);
void printAllChannels(void);
void printAlarmMasks(void);
void printPinAlarmMasks(void);
void printAlarmStatus(void);
void printPowerStateWord(void);
void printLimits(void);
void printTmcbPinVoltages(void);

void printBadArgument(void);
void printUnknownCommand(void);
void printVersion(void);	
void printPrompt(void);
void printWelcomeMessage(void);
void printHelp(void);

void channelsOff(void);
void channelsOn(void);
#define STATUS_ACTUAL_STATE (1 << 0)	///<Channel on
#define STATUS_UNEXPECTED	(1 << 1)	///<State not as expected
#define STATUS_I2C_ERROR	(1 << 4)	///<i2c error
#define STATUS_ALARM_0		(1 << 5)	///<Disabled by alarm 0 pin
#define STATUS_ALARM_1		(1 << 6)	///<Disabled by alarm 1 pin
#define STATUS_OVERCURRENT	(1 << 7)	///<Overcurrent flag
//void setFanSpeed( int spdRange); //added by Stanislav Chystiakov
//void readADC_Chan( int adcChnl);
#endif /* TELNET_PROTOCOL_H_ */