/**
 * @file timer_lib.C
 * @brief ATXMEGA timer support library
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Piotr Perek (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <asf.h>

#include "timer_lib.h"

#define PERIOD_10MS		1250
#define PERIOD_100MS	12500

volatile bool timer_100ms_flag = false;
volatile bool timer_30ms_flag = false;

/**
 * \brief 100 ms callback
 * 
 * \return void
 */
static void tc_overflow_callback(void)				//100 ms
{
	static uint8_t sec_counter = 0;
	sec_counter++;
	timer_100ms_flag = true;
	if(sec_counter == 2)
	{
		ioport_toggle_pin(MMC_LED);
		sec_counter = 0;
	}
}

//
//static void tc_cca_callback(void)					//10 ms
//{
	//static uint8_t sec_counter = 0;
	//sec_counter++;
	//uint16_t tmp = tc_read_cc(&TCC0, TC_CCA);
	//tc_write_cc(&TCC0, TC_CCA, (tmp+PERIOD_10MS) % PERIOD_100MS);
	//if (sec_counter == 3)
	//{
		//timer_30ms_flag = true;
		//sec_counter = 0;
	//}
//}



/**
 * \brief Initialize timer for 100 ms overflow period
 * 
 * \return void
 */
void timer_init(void)
{
	tc_enable(&TCC0);
	tc_set_wgm(&TCC0, TC_WG_NORMAL);
	tc_write_clock_source(&TCC0, TC_CLKSEL_DIV256_gc);
	tc_write_period(&TCC0, PERIOD_100MS);
//	tc_write_cc(&TCC0, TC_CCA, PERIOD_10MS);
//	tc_enable_cc_channels(&TCC0, TC_CCAEN);
	tc_set_overflow_interrupt_level(&TCC0, TC_INT_LVL_HI);
	tc_set_overflow_interrupt_callback(&TCC0, tc_overflow_callback);
//	tc_set_cca_interrupt_level(&TCC0, TC_INT_LVL_HI);
//	tc_set_cca_interrupt_callback(&TCC0, tc_cca_callback);
}